/**
 * NewTicketWizard module - SeTIC - UFSC - http://setic.ufsc.br/
 * Rodrigo Gonçalves - rodrigo.g@ufsc.br
 * 
 * Version 2017-12-19 - Conversion to JS Module, to support OTRS 6
 * 
 * 
 */

var Core = Core || {};
Core.Customer = Core.Customer || {};


Core.Customer.NewTicketWizard = (function (TargetNS) {
	var formAlpaca;
	var activeQueue;

	TargetNS.Init = function() {
		var formID = Core.Config.Get('NewTicketWizard.FormID');
		var public = Core.Config.Get('NewTicketWizard.Mode') === "Public";
		var sendMsg = Core.Language.Translate('Send');
		var cgiHandle = Core.Config.Get('CGIHandle');
		var queueID = Core.Config.Get('NewTicketWizard.QueueID', '');
		
		CKEDITOR.config.filebrowserUploadUrl = "/otrs/public.pl?Action=PictureUpload&responseType=json&FormID=" + formID;
		
		$.alpaca.setDefaultLocale((navigator.languages ? navigator.languages[0] : (navigator.language || navigator.userLanguage)).replace("-","_"));
				
		$.alpaca.registerMessages({
			"noneLabel": "-",
		});
		
		$("#ServiceID").change(function() {
			if ($("#ServiceID").val() != "") {
				if (public) {
					TargetNS.carregaFormPublic({"FormID": formID}, sendMsg, cgiHandle, ";QueueID=" + queueID, formID);
				} else {
					TargetNS.carregaForm({"FormID": formID}, sendMsg, cgiHandle, ";QueueID=" + queueID, formID);
				}
			
		   		
	   			$("#formPanel").show();
	   		} else {
	   			destruirForm();
	   			$("#serviceDescription").html("");
	   			$("#formPanel").hide();
	   		}  	
	  	});
	  	
	   	if ($("#ServiceID").has('[selected]')) {
	  		if ($("#ServiceID").val() != "") {
	  			if (public) {
	  				TargetNS.carregaFormPublic({"FormID": formID}, sendMsg, cgiHandle, ";QueueID=" + queueID, formID);
	  			} else {
	  				TargetNS.carregaForm({"FormID": formID}, sendMsg, cgiHandle, ";QueueID=" + queueID, formID);
	  			}
	  			$("#formPanel").show();  			  			
	  		}
	  	}
	  	
	  	$("#ResponsiveFooter").remove();
	}
	

	/**
	 * Build the form options (layout)
	 * @param fields
	 * @param sendText
	 * @returns {___anonymous58_421}
	 */
	TargetNS.buildOptions = function(fields, sendText, action) {
		return {    	
	    	"renderForm": true,
	        "fields": fields,        
	        "form": {
	        	"attributes" : {
	        		"action" : action,
	        		"method" : "post",
	        		"enctype": "multipart/form-data"
	        	},
	        	"buttons": {
	        		"submit": {
	        			"value" : sendText
	        		}
	        	},
	                "toggleSubmitValidState": false
	        	 
	        }
	    };
	}

	/**
	 * Build the form schema
	 * 
	 * @param schema
	 * @returns {___anonymous510_571}
	 */
	TargetNS.buildSchema = function(schema) {
		return {
	        "type": "object",
	        "properties": schema
	    };
	}

	TargetNS.postRenderCallback = function(form) {
		formOptions = form.options.fields;
		formAlpaca = form;
		
		if (form) {
			$('.selectpicker').selectpicker();
			$('.selectpicker').selectpicker('val', $("#ServiceID").val());
			//$("[name='service']").val();
			$("[name='FormID']").val(form.data.FormID);
	    	$("[name='submit']").css("font-size", "18px");
	    			
			form.form.registerSubmitHandler(function(e, form) {
	        	// validate the entire form (top control + all children)
	            form.validate(true);
	            
	            // draw the validation state (top control + all children)
	            form.renderValidationState(true);
	                    
	            // now display something
	            if (form.isFormValid()) {
	            	return true;
				} else {
					alert("Dados inválidos!");
					return false;
				}
	            e.stopPropagation();
	            return false;
	    	});
			
			
			// Fix CKEditor
			setTimeout(function() {
				for(var i in CKEDITOR.instances) {
					var editor = CKEDITOR.instances[i];
					editor.config.filebrowserUploadUrl = "/otrs/public.pl?Action=PictureUpload&FormID=" + form.data.FormID;
				}			
			}, 2000);
		}
		
		TargetNS.carregaCustomProps(activeQueue);
	};


	TargetNS.postRenderCallbackPublic = function(form) {
		formOptions = form.options.fields;
		formAlpaca = form;
		
		if (form) {
			$('.selectpicker').selectpicker();
			//$("[name='service']").val($("#ServiceID").val());
			$("[name='FormID']").val(form.data.FormID);		
	    	$("[name='submit']").css("font-size", "18px");
	    			
			form.form.registerSubmitHandler(function(e, form) {
	        	// validate the entire form (top control + all children)
	            form.validate(true);
	            
	            // draw the validation state (top control + all children)
	            form.renderValidationState(true);
	                    
	            // now display something
	            if (form.isFormValid()) {
	            	return true;
				} else {
					alert("Dados inválidos!");
					return false;
				}
	            e.stopPropagation();
	            return false;
	    	});
			
			// Fix CKEditor
			setTimeout(function() {
				for(var i in CKEDITOR.instances) {
					var editor = CKEDITOR.instances[i];
			        editor.config.filebrowserUploadUrl = "/otrs/public.pl?Action=PictureUpload&FormID=" + form.data.FormID;
				}
			
			}, 2000);		
		}
		
		TargetNS.carregaCustomPropsPublic(activeQueue);
	};

	TargetNS.carregaCustomPropsPublic = function(queue) {
		if (! queue) {
			queue = "";
		}
		
		var url = "/otrs/public.pl?Action=NewTicketWizardPublic;Subaction=GetFormCustomProps" + queue;
		
		url += ";ServiceID=" + $("#ServiceID").val();
		
		$.getScript(url, function(data) {		
	    }).fail(function(jqXHR, textStatus, errorThrown) {
	    alert(textStatus);
	  });
	}

	TargetNS.carregaCustomProps = function(queue) {
		if (! queue) {
			queue = "";
		}	
		var url = "/otrs/customer.pl?Action=NewTicketWizard;Subaction=GetFormCustomProps" + queue;
		
		url += ";ServiceID=" + $("#ServiceID").val();
		
		$.getScript(url, function(data) {		
	    }).fail(function(jqXHR, textStatus, errorThrown) {
	    alert(textStatus);
	  });
	}
	    
	TargetNS.carregaForm = function(dataForm, sendText, action, queue, formID) {	
		var url = "/otrs/customer.pl?Action=NewTicketWizard;Subaction=GetFormJSON" + queue;
		
		if (!(typeof formAlpaca === 'undefined')) {
			formAlpaca.destroy();
		} 
		
		activeQueue = queue;	
		url += ";ServiceID=" + $("#ServiceID").val();
		url += ";FormID=" + formID;
		
		$.getJSON(url, function(data) {		
			$("#serviceDescription").html(data[2]);
			$("#serviceDescription").show();
			
			$("#formTicket").alpaca({
	       		"data": dataForm,
	        	"schema": TargetNS.buildSchema(data[0]),
	        	"options": TargetNS.buildOptions(data[1], sendText, action),			
	        	"postRender": TargetNS.postRenderCallback,
				"view": "bootstrap-create",
				"ui": "bootstrap-ui"
	    	}); 
	    	
	    }).fail(function(jqXHR, textStatus, errorThrown) {
	    	alert("Erro ao recuperar: " + url + " => " + textStatus);
	  });
	}

	TargetNS.reloadServices = function(send, cgihandle) {
		var url = "/otrs/customer.pl?Action=NewTicketWizard;Subaction=GetServices;QueueID=" + $("#QueueID").val();
		
		$.get(url, function(data) {		
			$("#serviceDIV").html(data);
			
			$("#ServiceID").change(function() {
		   		carregaForm(null, send,cgihandle, ";QueueID=" + $("#QueueID").val());
		  	});
			
			$("#formTicket").html("");
			$("#serviceDescription").hide();

			
	    }).fail(function(jqXHR, textStatus, errorThrown) {
	    	alert(textStatus);
	  });
	}

	TargetNS.carregaFormPublic = function(dataForm, sendText, action, queue, formID) {
		var url = "/otrs/public.pl?Action=NewTicketWizardPublic;Subaction=GetFormJSON" + queue;
		
		if (!(typeof formAlpaca === 'undefined')) {
			formAlpaca.destroy();
		}
		
		activeQueue = queue;	
		url += ";ServiceID=" + $("#ServiceID").val();
		url += ";FormID=" + formID;
		
		$.getJSON(url, function(data) {		
			$("#serviceDescription").html(data[2]);
			$("#serviceDescription").show();
			
			var form = $("#formTicket").alpaca({
	       		"data": dataForm,
	        	"schema": TargetNS.buildSchema(data[0]),
	        	"options": TargetNS.buildOptions(data[1], sendText, action),			
	        	"postRender": TargetNS.postRenderCallbackPublic,
				"view": "bootstrap-create",
				"ui": "bootstrap-ui"
	    	}); 
			
			
	    	
	    }).fail(function(jqXHR, textStatus, errorThrown) {
	    	alert("Erro ao recuperar: " + url + " => " + textStatus);
	  });
	}

	TargetNS.destruirForm = function() {
		$("#formTicket").alpaca().destroy();
	}
	
	Core.Init.RegisterNamespace(TargetNS, 'APP_MODULE');
	
	return TargetNS;
	
}(Core.Customer.NewTicketWizard || {}));






