# --
# This software comes with ABSOLUTELY NO WARRANTY. For details, see
# the enclosed file COPYING for license information (AGPL). If you
# did not receive this file, see http://www.gnu.org/licenses/agpl.txt.
#
# Frontend module for creating custom new ticket interfaces without user login - SeTIC - UFSC - http://setic.ufsc.br/
# Rodrigo Gonçalves - rodrigo.g@ufsc.br
# 
# Copyright (C) SeTIC - UFSC - http://setic.ufsc.br/
# Version 2015-08-01 - Support for OTRS 4.0.3
# Version 2015-06-25 - Support for restricted services and HTML formatting of form fields
# Version 2017-06-16 - Support for bootstrap and logged in user identification
# Version 2017-06-20 - Code refactoring
# Version 2017-12-18 - Support for OTRS 6 and module publishing
#
# --
package Kernel::Modules::NewTicketWizardPublic;

use strict;
use warnings;
use Kernel::System::VariableCheck qw(:all);
use Data::UUID;
use Data::Dumper;
use URI::Escape;
 
our @ObjectDependencies = (
"Kernel::System::TicketWizard",
"Kernel::System::Web::Request",
"Kernel::Output::HTML::Layout",
"Kernel::Config"
 );
 
sub new {
	my ( $Type, %Param ) = @_;

	# allocate new hash for object
	my $Self = {%Param};
	bless( $Self, $Type );

	return $Self;
}


sub Run {
	my ( $Self, %Param ) = @_;
		
	my $ParamObject = $Kernel::OM->Get("Kernel::System::Web::Request");
	my $LayoutObject = $Kernel::OM->Get("Kernel::Output::HTML::Layout");
	my $ConfigObject = $Kernel::OM->Get("Kernel::Config");
	my $TicketWizard = $Kernel::OM->Get("Kernel::System::TicketWizard");

	$LayoutObject->AddJSData(
    	Key   => 'NewTicketWizard.Mode',
    	Value => "Public",
	);

	# If a ticket was received, it means the user is already authenticated
	$Self->Debug("Checking if user was authenticated");
	if ( $ParamObject->GetParam( Param => "ticket" ) ) {
		my $retorno = $LayoutObject->Redirect( OP => "Action=NewTicketWizard", );
		$retorno =~ s/public/customer/g;
		return $retorno;
	}

	# If an authenticate parameter was received, redirect to CAS gateway
	$Self->Debug("Checking if authenticate parameter was received");
	if ( $ParamObject->GetParam( Param => "authenticate" ) ) {
		$Self->Debug("Authenticate parameter received");
		my $url =
		    $ConfigObject->Get('Customer::AuthModule::CAS::CASUrl')
		  . "/login?gateway=true&service="
		  . "https://"
		  . $ENV{SERVER_NAME}
		  . $ENV{SCRIPT_NAME}
		  . "?Action=NewTicketWizardPublic";
		return $LayoutObject->Redirect( ExtURL => $url, );
	}
		  
	# If no authentication was possible, proceed with public ticket opening
	$Self->Debug("Public ticket started");
	$Self->Debug("Subaction => " . $ParamObject->GetParam( Param => "Subaction" ));

	
	if ( $ParamObject->GetParam( Param => "UploadFileCmd" )) {
		$Self->Debug("File upload instruction received");
		return $TicketWizard->UploadFile();
	} elsif ( $ParamObject->GetParam( Param => "Subaction" ) ) {
		if ( $ParamObject->GetParam( Param => "Subaction" ) eq "CreateTicket" ) {
			$Self->Debug("Create ticket action received");
			return $TicketWizard->CreateTicket(Public => 1);
		}
		elsif ( $ParamObject->GetParam( Param => "Subaction" ) eq "GetFormJSON" ) {
			$Self->Debug("Service form requested received");
			return $TicketWizard->GetFormJSON( Public=> 1, ServiceID => $ParamObject->GetParam( Param => "ServiceID" ) );
		} elsif ( $ParamObject->GetParam( Param => "Subaction" ) eq "GetFormCustomProps" ) {
			$Self->Debug("Service form custom properties (script) requested");
			return $TicketWizard->GetFormCustomProps( Public=> 1, ServiceID => $ParamObject->GetParam( Param => "ServiceID" ) );
		} 
	} else {
		$Self->Debug("Default action - open service choosing form");
		return $TicketWizard->OpenForm(Public => 1);
	}
}

sub Debug {
    my $Self = shift;
    my $msg = shift;
	$Kernel::OM->Get("Kernel::System::Log")->Log( Priority => 'debug', Message => $msg );
}

1;
